/* 
 * Motif Tools Library, Version 2.0
 * $Id: XmToggleB.c,v 2.9 1994/07/04 03:03:50 david Exp $
 * 
 * Written by David Flanagan.
 * Copyright (c) 1992, 1993, 1994 by Dovetail Systems.
 * All Rights Reserved.  See the file COPYRIGHT for details.
 * This is not free software.  See the file SHAREWARE for details.
 * There is no warranty for this software.  See NO_WARRANTY for details.
 */

#include <Xmt/Xmt.h>
#include <Xmt/WidgetType.h>
#include <Xmt/QuarksP.h>
#include <Xm/ToggleB.h>

#if NeedFunctionPrototypes
static void setvalue(Widget w, XtPointer address, XrmQuark type, Cardinal size)
#else
static void setvalue(w, address, type, size)
Widget w;
XtPointer address;
XrmQuark type;
Cardinal size;
#endif
{
    int state;

    if ((type != XmtQBoolean) &&
	(type != XmtQBool) &&
	(type != XmtQCardinal) &&
	(type != XmtQEnum) &&
	(type != XmtQInt) &&
	(type != XmtQShort) &&
	(type != XmtQUnsignedChar)) {
	XmtWarningMsg("XmtDialogSetDialogValues", "toggleb",
		      "Type mismatch: Widget '%s':\n\tCan't set widget value from a resource of type '%s'; scalar type expected.",
		      XtName(w), XrmQuarkToString(type));
	return;
    }

    if (size == sizeof(unsigned char))
	state = *(unsigned char *)address;
    else if (size == sizeof(unsigned short))
	state = *(unsigned short *)address;
    else if (size == sizeof(unsigned int))
	state = *(unsigned int *)address;
    else return;
    
    XmToggleButtonSetState(w, state, False);
}

#if NeedFunctionPrototypes
static void getvalue(Widget w, XtPointer address, XrmQuark type, Cardinal size)
#else
static void getvalue(w, address, type, size)
Widget w;
XtPointer address;
XrmQuark type;
Cardinal size;
#endif
{
    Boolean state = XmToggleButtonGetState(w);

    if ((type != XmtQBoolean) &&
	(type != XmtQBool) &&	
	(type != XmtQCardinal) &&
	(type != XmtQEnum) &&
	(type != XmtQInt) &&
	(type != XmtQShort) &&
	(type != XmtQUnsignedChar)) {
	XmtWarningMsg("XmtDialogGetDialogValues", "toggleb",
		      "Type mismatch: Widget '%s':\n\tCan't set state on resource of type '%s'; scalar type expected.",
		      XtName(w), XrmQuarkToString(type));
	return;
    }

    if (size == sizeof(unsigned char))
	*(unsigned char *)address = state;
    else if (size == sizeof(unsigned short))
	*(unsigned short *)address = state;
    else if (size == sizeof(unsigned int))
	*(unsigned int *)address = state;
}

static XmtWidgetType widget = {
    "XmToggleButton",
    NULL,
    XmCreateToggleButton,
    setvalue,
    getvalue
};

#if NeedFunctionPrototypes
void XmtRegisterXmToggleButton(void)
#else
void XmtRegisterXmToggleButton()
#endif
{
    _XmtInitQuarks();
    XmtRegisterWidgetTypes(&widget, 1);
}

